function [leghandle,labelhandles]=legend(varargin)
%LEGEND Graph legend.
%   LEGEND(string1,string2,string3, ...) puts a legend on the current plot
%   using the specified strings as labels. LEGEND works on line graphs,
%   bar graphs, pie graphs, ribbon plots, etc.  You can label any
%   solid-colored patch or surface object.  The fontsize and fontname for
%   the legend strings matches the axes fontsize and fontname.
%
%   LEGEND(H,string1,string2,string3, ...) puts a legend on the plot
%   containing the handles in the vector H using the specified strings as
%   labels for the corresponding handles.
%
%   LEGEND(M), where M is a string matrix or cell array of strings, and
%   LEGEND(H,M) where H is a vector of handles to lines and patches also
%   works.
%
%   LEGEND(AX,...) puts a legend on the axes with handle AX.
%
%   LEGEND OFF removes the legend from the current axes.
%   LEGEND(AX,'off') removes the legend from the axis AX.
%
%   LEGH = LEGEND returns the handle to legend on the current axes or
%   empty if none exists.
%
%   LEGEND with no arguments refreshes all the legends in the current
%   figure (if any).  LEGEND(LEGH) refreshes the specified legend.
%
%   LEGEND(...,Pos) places the legend in the specified
%   location:
%       0 = Automatic "best" placement (least conflict with data)
%       1 = Upper right-hand corner (default)
%       2 = Upper left-hand corner
%       3 = Lower left-hand corner
%       4 = Lower right-hand corner
%      -1 = To the right of the plot
%
%   To move the legend, press the left mouse button on the legend and drag
%   to the desired location. Double clicking on a label allows you to edit
%   the label.
%
%   [LEGH,OBJH] = LEGEND(...) returns a handle LEGH to the legend axes and
%   vector OBJH containing the text, line, and patch handles.
%
%   LEGEND will try to install a ResizeFcn on the figure if it hasn't been
%   defined before.  This resize function will try to keep the legend the
%   same size.
%
%   Examples:
%       x = 0:.2:12;
%       plot(x,bessel(1,x),x,bessel(2,x),x,bessel(3,x));
%       legend('First','Second','Third');
%       legend('First','Second','Third',-1)
%
%       b = bar(rand(10,5),'stacked'); colormap(summer); hold on
%       x = plot(1:10,5*rand(10,1),'marker','square','markersize',12,...
%                'markeredgecolor','y','markerfacecolor',[.6 0 .6],...
%                'linestyle','-','color','r','linewidth',2); hold off
%       legend([b,x],'Carrots','Peas','Peppers','Green Beans',...
%                 'Cucumbers','Eggplant')       
%
%   See also PLOT.

%   D. Thomas 5/6/93
%             9/6/95  
%   Rich Radke 6/17/96 Latest Update
%   Copyright (c) 1984-98 by The MathWorks, Inc.
%   $Revision: 5.53 $  $Date: 1998/12/10 22:07:24 $

%   Private syntax:
%
%     LEGEND('DeleteLegend') is called from the deleteFcn to remove the legend.
%     LEGEND('EditLegend',h) is called from MOVEAXIS to edit the legend labels.
%     LEGEND('ShowLegendPlot') is called from MOVEAXIS to set the gco to
%     the plot the legend goes with.
%     LEGEND('ResizeLegend') is called from the resizeFcn to resize the legend.
%
%   Obsolete syntax:
%
%     LEGEND(linetype1,string1,linetype2,string2, ...) specifies
%     the line types/colors for each label.
%     Linetypes can be any valid PLOT linetype specifying color,
%     marker type, and linestyle, such as 'g:o'.  

narg = nargin;
isresize(0);

%--------------------------
% Parse inputs
%--------------------------

% Determine the legend parent axes (ha) is specified
if narg > 0 & ~isempty(varargin{1}) & ishandle(varargin{1}) & ...
	strcmp(get(varargin{1}(1),'type'),'axes') % legend(ax,...)
    ha = varargin{1}(1);
    varargin(1) = []; % Remove from list
    narg = narg - 1;
    if strcmp(get(ha,'Tag'),'legend') % Use the parent
      ud = get(ha,'userdata');
      if isfield(ud,'PlotHandle')
         ha = ud.PlotHandle;
      else
        warning('Can''t put a legend on a legend.')
        if nargout>0, leghandle = []; labelhandles = []; end
        return
      end
    end
else
  ha = [];
end

if narg==0 % legend
  if nargout==1, % h = legend
    if isempty(ha)
      leghandle = find_legend(find_gca);
    else
      leghandle = find_legend(ha);
    end
  elseif nargout==0 % legend
    if isempty(ha)
      update_all_legends
    else
      update_legend(find_legend(ha));
    end
  else % [h,objh] = legend
    if isempty(ha)
      [leghandle,labelhandles] = find_legend(find_gca);
    else
      [leghandle,labelhandles] = find_legend(ha);
    end
  end
  return
elseif narg==1 & strcmp(varargin{1},'DeleteLegend'), % legend('DeleteLegend')
  % Should only be called by the deleteFcn
  delete_legend(gcbo)
  if nargout>0, error('Too many outputs.'); end
  return
elseif narg==1 & strcmp(varargin{1},'ResizeLegend'), % legend('ResizeLegend')
  isresize(1);
  resize_all_legends(gcbf)
  isresize(0);
  if nargout>0, error('Too many outputs.'); end
  return
elseif narg==1 & strcmp(varargin{1},'off'), % legend('off') or legend(AX,'off')
  if isempty(ha)
    delete_legend(find_legend(find_gca))
  else
    delete_legend(find_legend(ha))
  end   
  if nargout>0, error('Too many outputs.'); end
  return
elseif narg==1 & strcmp(varargin{1},'ShowLegendPlot')
  show_plot
  return
elseif narg==2 & strcmp(varargin{1},'EditLegend')
  edit_legend(varargin{2})
  return
elseif narg==1 & islegend(varargin{1}) % legend(legh)
  [hl,labelhandles] = update_legend(varargin{1});
  if nargout>0, leghandle = hl; end
  return
end

% Look for legendpos code
if isa(varargin{end},'double')
  legendpos = varargin{end};
  varargin(end) = [];
else
  legendpos = [];
end

% Determine the active children (kids) and the strings (lstrings)
if narg < 1
  error('Not enough input arguments.');
elseif ishandle(varargin{1}) % legend(h,strings,...)
  kids = varargin{1};
  if isempty(ha)
    ha=get(varargin{1}(1),'Parent');
    if ~strcmp(get(ha,'type'),'axes'),
      error('Handle must be an axes or child of an axes.');
    end
  end
  if narg==1, error('A string must be supplied for each handle.'); end
  lstrings = getstrings(varargin(2:end));
else % legend(strings,...) or legend(linetype,string,...)
  if isempty(ha), ha=find_gca; end
  kids = getchildren(ha);
  lstrings = getstrings(varargin);
end

% Set default legendpos
if isempty(legendpos)
  if ~isequal(get(ha,'view'),[0 90])
    legendpos = -1;  % To the right of axis is default for 3-D
  else
    legendpos = 1;   % Upper right is default for 2-D
  end
end

% Remove any existing legend on this plot 
hl = find_legend;
if ~isempty(hl),
  ud = get(hl,{'userdata'});
  for i=1:length(ud)
    if isfield(ud{i},'PlotHandle') & ud{i}.PlotHandle == ha
      delete_legend(hl)
    end
  end
end

if length(kids)==0,
  warning('Plot empty.')
  if nargout>0
    leghandle = []; labelhandles = [];
  end
  return
end

[hl,labelhandles] = make_legend(find_legend(ha),ha,kids,lstrings,legendpos);

if nargout > 0,
    leghandle=hl;
end

%
% end of legend.m
%
