%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes multivariate LM tests for p versus p+k lags  %
%                                                                %
%          Function: lmlagtest                                   %
%                                                                %
%          Input:                                                %
%                       epsilon: nxT matrix with residuals       %
%                       x0: Tpxn matrix                          %
%                       x1: Tpxd matrix                          %
%                       x2: Tpxq matrix                          %
%                       x1_r: Tpxdr matrix                       %
%                       p: number of lags under the null         %
%                       pe: number of exogenous lags under       %
%                           the null                             %
%                       k: number of additional lags under       %
%                           the alternative                      %
%                       q1: number of exogenous I(1) variables   %
%                       q0: number of exogenous I(0) variables   %
%                       beta: n+q1+dcointxr matrix with          %
%                           cointegration vectors                %
%                       t: degrees of freedom from original      %
%                           estimates                            %
%                       ind: 0-1 indicator to use restrictions   %
%                           on alpha under the null and the      %
%                           alternative (default=0)              %
%                       moddir: path to G-modelfile.rest         %
%                           files (default=empty string)         %
%                       modfile: name of model (default is       %
%                           empty string)                        %
%                                                                %
%          Output:                                               %
%          		          results: matrix with results             %
%                       df: matrix with degrees of freedom       %
%                       mycorrect: indicator if F-type test      %
%                           has been computed                    %
%                                                                %
%          This version:        January 20, 2005                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [results,df,mycorrect] = lmlagtest(epsilon,x0,x1,x1_r,x2,p,pe,k,q1,q0,beta,t,ind,moddir,modfile,c_ind,Psi,alpha,Z1,Z2,d)

if nargin<13;
   ind = 0;
   moddir = '';
   modfile = '';
end;

if nargin<16;
   c_ind = 0;
   Psi = [];
   alpha = [];
   Z1 = [];
   Z2 = [];
   d = 0;
end;

curr_ini = find_inisettings;

%
% Preliminaries
%

if isempty(x2)==1;
   [w0,w1,w2,w1t]=vecmdata(x0,p+k,x1);
else;
   [w0,w1,w2,w1t]=vecmdata(x0,p+k,x1,x2,q1,pe+k);
end;

if isempty(x1_r)==0;
   w1 = [w1;x1_r(k+1:size(x1_r,1),:)'];
end;

[n Torig] = size(epsilon);
T = size(w0,2);
r = size(beta,2);
t = t-(Torig-T);

if r>0;
   w = [w2;beta'*w1];
else;
   w = w2;
end;

epsilon = epsilon(:,(Torig-T+1):Torig);
Omega = (1/T)*epsilon*epsilon';

nr = n*k*(n+q1+q0);
nreg = size(w,1);

if nr>=(t-(k*(n+q1+q0)));
   curr_ini.sscorrect = 0;
end;

inv_Omega = Omega\eye(n);

if c_ind==1;
   file = strcat(moddir,'C-',int2str(n),'x',int2str(r),'-',modfile,'.rest');
   F = loaddata(file);
   Psi = [Psi zeros(n,n*k)];
   [Rmat_C,rvec_C] = rmatrix(F,n,n);
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,w1,w2,p+k,d,0);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,w1,w2,p+k,d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,p+k,pe+k,r,d,q1,q0);
   Rmat_C = Rmat_C*kron(xi',C);
   if rank(Rmat_C,eps)<size(Rmat_C,1);
      [red_rest,use_rest] = getdelvars(Rmat_C',[]);
      Rmat_C = Rmat_C(use_rest,:);
   end;
else;
   Rmat_C = [];
end;

if ind==0;
   if isempty(Rmat_C)==0;
      H = null(Rmat_C);
      Score_theta = H'*vec(inv_Omega*epsilon*w');
      Sigma_theta = H'*kron(w*w',inv_Omega)*H;
      nr2 = size(Sigma_theta,1);
      Sigma_theta = eye(nr2)/Sigma_theta;
   else;
      inv_ww = eye(nreg)/(w*w');
      Sigma_theta = kron(inv_ww,Omega);
      Score_theta = vec(inv_Omega*epsilon*w');
   end;
else;
   %
   % Get the restrictions file
   %
   G = [];
   file = strcat(moddir,'G-',int2str(n),'x',int2str(r),'-',modfile,'.rest');
   if fileexist(file)==1;
      G = loaddata(file);
   end;
   if isempty(G)==0;
      [Rmat,rvec] = rmatrix(G,n,r);
      dfa = size(G,1);
      Rmat = [zeros(dfa,(nreg-r)*n) Rmat];
      if isempty(Rmat_C)==0;
         Rmat = [Rmat;Rmat_C];
         if rank(Rmat)<size(Rmat,1);
            [red_rest,use_rest] = getdelvars(Rmat',[]);
            Rmat = Rmat(use_rest,:);
         end;
      end;
      H = null(Rmat);
      Score_theta = H'*vec(inv_Omega*epsilon*w');
      Sigma_theta = H'*kron(w*w',inv_Omega)*H;
      nr2 = size(Sigma_theta,1);
      Sigma_theta = eye(nr2)/Sigma_theta;
   else;
      if isempty(Rmat_C)==0;
         H = null(Rmat_C);
      else;
         H = eye(nreg*n);
      end;
      Score_theta = H'*vec(inv_Omega*epsilon*w');
      Sigma_theta = H'*kron(w*w',inv_Omega)*H;
      nr2 = size(Sigma_theta,1);
      Sigma_theta = eye(nr2)/Sigma_theta;
%      inv_ww = eye(nreg)/(w*w');
%      Sigma_theta = kron(inv_ww,Omega);
      % Score_theta = kron(eye(nreg),inv_Omega)*vec(epsilon*w');
%      Score_theta = vec(inv_Omega*epsilon*w');
   end;
end;

LM = Score_theta'*Sigma_theta*Score_theta;
if (curr_ini.bartserial==1)&(isalmostequal(q1+q0,0)==1);
   k0 = p;
   k1 = p+k;
   bc = 1+((1/(2*T*k)))*( (n+1)*k + n*((k1*k1)-(k0*k0)) )+(1/(T*k*n*n))*( n*(floor(k1/2)-floor(k0/2)) -(n/2)*( k1*(k1-1) -k0*(k0-1)));
   LM = LM/bc;
end;
if curr_ini.sscorrect==0;
   df = nr;
   pval = chis_cdf(LM,nr);
   results = [LM 1-pval];
else;
   df = [nr t-(k*(n+q1+q0))];
   LM = ((t-(k*(n+q1+q0)))/(T*nr))*LM;
   pval = fdis_cdf(LM,df(1,1),df(1,2));
   results = [LM 1-pval];
end;

if nargout>2;
   mycorrect = curr_ini.sscorrect;
end;

%
% end of lmlagtest.m
%
