function info = legend_info(ha,hf,Kids,lstrings);
%LEGEND_INFO Get legend info from parent axes, Kids, and strings
%   INFO = LEGEND_INFO(HA,KIDS,STRINGS) returns a structure array containing
%      objtype  -- Type of object 'line', 'patch', or 'surface'
%      label    -- label string
%      linetype -- linetype;
%      edgecol  -- edge color
%      facecol  -- face color
%      lnwidth  -- line width
%      marker   -- marker
%      marksize -- markersize
%      markedge -- marker edge color
%      markface -- marker face color (not used for 'line')

defaultlinestyle = get(hf,'defaultlinelinestyle');
defaultlinecolor = get(hf,'defaultlinecolor');
defaultlinewidth = get(hf,'defaultlinelinewidth');
defaultlinemarker = get(hf,'defaultlinemarker');
defaultlinemarkersize = get(hf,'defaultlinemarkersize');
defaultlinemarkerfacecolor = get(hf,'defaultlinemarkerfacecolor');
defaultlinemarkeredgecolor = get(hf,'defaultlinemarkeredgecolor');
defaultpatchfacecolor = get(hf,'defaultpatchfacecolor');

linetype = {};
edgecol = {};
facecol = {};
lnwidth = {};
marker = {};
marksize = {};
markedge = {};
markface = {};

% These 8 variables are the important ones.  The only ambiguity is
% edgecol/facecol.  For lines, edgecol is the line color and facecol
% is unused.  For patches, edgecol/facecol mean the logical thing.

Kids = Kids(:);  %  Reshape so that we have a column vector of handles.
lstrings = lstrings(:);

% Check for valid handles
nonhandles = ~ishandle(Kids);
if any(nonhandles)
%  warning('Some invalid handles were ignored.')
  Kids(nonhandles) = [];
end
if ~isempty(Kids)
badhandles = ~(strcmp(get(Kids,'type'),'patch') | ...
               strcmp(get(Kids,'type'),'line')  | ...
               strcmp(get(Kids,'type'),'surface'));
if any(badhandles)
%  warning(['Some handles to non-lines and/or non-solid color',...
%           ' objects were ignored.'])
  Kids(badhandles) = [];
end
end

% Look for obsolete syntax label(...,LineSpec,Label,LineSpec,Label)
% To reduce the number of false hits, we require at least one
% line type in the list
obsolete = 0;
if rem(length(lstrings),2)==0,
  for i=1:2:length(lstrings)
    if isempty(lstrings{i}), % Empty lineSpec isn't obsolete syntax 
      obsolete = 0;
      break
    end
    [L,C,M,msg] = colstyle(lstrings{i});
    if ~isempty(msg), % If any error parsing LineSpec 
       obsolete = 0;
       break
     end
    if ~isempty(L), obsolete = 1; end
  end
end

if obsolete
  warning(sprintf(['The syntax LEGEND_FCN(linetype1,string1,linetype2,',...
           'string2, ...) \n is obsolete.  Use LEGEND_FCN(H,string1,',...
           'string2, ...) instead, \n where H is a vector of handles ',...
           'to the objects you wish to label.']))

  % Every other argument is a linespec

  % Right now we don't check to see if a corresponding linespec is
  % actually present on the graph, we just draw it anyway as a 
  % simple line with properties color, linestyle, 1-char markertype.
  % No frills like markersize, marker colors, etc.  Exception: if
  % a patch is present with facecolor = 'rybcgkwm' and the syntax
  % legend_fcn('g','label') is used, a patch shows up in the legend
  % instead.  Since this whole functionality is being phased out and
  % you can do better things using handles, the legend may not look
  % as nice using this option.
  
  objtype = {};

  % Check for an even number of strings
  if rem(length(lstrings),2)~=0
    error('Invalid legend syntax.')
  end

  for i=1:2:length(lstrings)        
    lnstr=lstrings{i};
    [lnt,lnc,lnm,msg] = colstyle(lnstr);

    if (isempty(msg) & ~isempty(lnstr)) % Valid linespec
      % Check for line style
      if (isempty(lnt))
        linetype=[linetype,{defaultlinestyle}];
      else
        linetype=[linetype,{lnt}];
      end
      % Check for line color
      if (isempty(lnc))
        edgecol=[edgecol,{defaultlinecolor}];
        facecol=[facecol,{defaultpatchfacecolor}];
        objtype = [objtype,{'line'}];
      else   
        colspec = ctorgb(lnc);
        edgecol=[edgecol,{colspec}];
        facecol=[facecol,{colspec}];
        if ~isempty(findobj('type','patch','facecolor',colspec)) | ...
           ~isempty(findobj('type','surface','facecolor',colspec))
          objtype = [objtype,{'patch'}];
        else
          objtype = [objtype,{'line'}];
        end
      end
      % Check for marker
      if (isempty(lnm)),
          marker=[marker,{defaultlinemarker}];
      else
          marker=[marker,{lnm}];
      end
      % Set remaining properties
      lnwidth = [lnwidth,{defaultlinewidth}];
      marksize = [marksize,{defaultlinemarkersize}];
      markedge = [markedge,{defaultlinemarkeredgecolor}];
      markface = [markface,{defaultlinemarkerfacecolor}];
    else
      % Set everything to defaults
      linetype=[linetype,{defaultlinestyle}];
      edgecol=[edgecol,{defaultlinecolor}];
      facecol=[facecol,{defaultpatchfacecolor}];
      marker=[marker,{defaultlinemarker}];
      lnwidth = [lnwidth,{defaultlinewidth}];
      marksize = [marksize,{defaultlinemarkersize}];
      markedge = [markedge,{defaultlinemarkeredgecolor}];
      markface = [markface,{defaultlinemarkerfacecolor}];
      objtype = [objtype,{'line'}];
    end
  end
  lstrings = lstrings(2:2:end);

else % Normal syntax
  objtype = get(Kids,{'type'});
  nk = length(Kids);
  nstack = length(lstrings);
  n = min(nstack,nk);

  % Treat empty strings as a single space
  for i=1:nstack
    if isempty(lstrings{i})
      lstrings{i} = ' ';
    end
  end
  
  % Truncate kids if necessary to match the number of strings
  objtype = objtype(1:n);
  Kids = Kids(1:n);

  for i=1:n
    linetype = [linetype,get(Kids(i),{'LineStyle'})];
    if strcmp(objtype{i},'line')
      edgecol = [edgecol,get(Kids(i),{'Color'})];            
      facecol = [facecol,{'none'}];
    elseif strcmp(objtype{i},'patch') | strcmp(objtype{i},'surface')
      [e,f] = patchcol(Kids(i));
      edgecol = [edgecol,{e}];
      facecol = [facecol,{f}];
    end
    lnwidth = [lnwidth,get(Kids(i),{'LineWidth'})];
    marker = [marker,get(Kids(i),{'Marker'})];
    marksize = [marksize,get(Kids(i),{'MarkerSize'})];
    markedge = [markedge,get(Kids(i),{'MarkerEdgeColor'})];
    markface = [markface,get(Kids(i),{'MarkerFaceColor'})];
  end

  if n < nstack,     % More strings than handles
    objtype(end+1:nstack) = {'none'};
    linetype(end+1:nstack) = {'none'};
    edgecol(end+1:nstack) = {'none'};
    facecol(end+1:nstack) = {'none'};
    lnwidth(end+1:nstack) = {defaultlinewidth};
    marker(end+1:nstack) = {'none'};
    marksize(end+1:nstack) = {defaultlinemarkersize};
    markedge(end+1:nstack) = {'auto'};
    markface(end+1:nstack) = {'auto'};
  end
end

% Limit markersize to axes fontsize
fonts = get(ha,'fontsize');
marksize([marksize{:}]' > fonts & strcmp(objtype(:),'line')) = {fonts};  
marksize([marksize{:}]' > fonts/2 & strcmp(objtype(:),'patch')) = {fonts/2};  

% Package everything into the info structure
info = struct('objtype',objtype(:),'label',lstrings(:),...
              'linetype',linetype(:),'edgecol',edgecol(:),...
             'facecol',facecol(:),'lnwidth',lnwidth(:),'marker',marker(:),...
             'marksize',marksize(:),'markedge',markedge(:),'markface',markface(:));
 

%
% end of legend_info.m
%
