%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for creating data matrices            %
%          for VAR model in VECM form                            %
%                                                                %
%          Function: vecmdata                                    %
%                                                                %
%          Input:                                                %
%                        x0: Txn containing all endogenous       %
%                           variables in levels                  %
%                        p: the lag length for the endogenous    %
%                           variables                            %
%                        x1: Txd containing all deterministic    %
%                           variables                            %
%                        x2: Txq containing all exogenous        %
%                           I(1) and I(0) variables in levels    %
%                        q1: the number of I(1) varibles in x2   %
%                            ordered in the first q1 columns     %
%                        pe: lag length for exogenous I(1) and   %
%                           I(0) variables                       %
%                                                                %
%          Output:                                               %
%                        Z0: nx(T-p) containing all endogenous   %
%                            variables in first diffs            %
%                        Z1: n+q1x(T-p) containing first lag of  %
%                            all endogenous and I(1) exogenous   %
%                            variables in levels                 %
%                        Z2: n(p-1)+qp+dx(T-p) containing        %
%                            lags of endogenous first diffs,     %
%                            contemporaneous and lags of first   %
%                            diffs of I(1) and levels of I(0)    %
%                            exogenous variables, and            %
%                            deterministic variables             %
%                        Z1t: n+q1x(T-p) matrix containing all   %
%                            endogenous and I(1) exogenous       %
%                            variables in levels                 %
%                                                                %
%          This version: 	      February 1, 2005                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Z0,Z1,Z2,Z1t] = vecmdata(x0,p,x1,x2,q1,pe)

T = size(x0,1);
n = size(x0,2);

%
% Creating the vector of deterministic variables
% If none are included in the function, a vector
% of constants is created
%

if size(x1,1)>0;
   d = size(x1,2);
else;
   d = 0;
end;

%
% Checking if there are any exogenous stochastic
% variables
%

if nargin>=4;
   q = size(x2,2);
else;
   q = 0;
   pe = 0;
end;

if nargin<5;
   q1 = 0;
   q2 = 0;
end;

dx0 = x0(2:T,:)-x0(1:T-1,:);

if size(x1,1)>0;
   Z2 = x1(max(p,pe)+1:T,:)';
else;
   Z2 = [];
end;

if q>=1;
   q2=q-q1;
   if q1>=1;
      dx2 = x2(2:T,1:q1)-x2(1:T-1,1:q1);
   end;
   if q2>=0;
      x3 = x2(:,q1+1:q);
   end;
end;

%
% Creating the output matrices
%

if max(p,pe)<2;
   Z0 = dx0';
   Z1 = x0(1:T-1,:)';
   Z1t = x0(2:T,:)';
   if q1>0;
      if pe>0;
         Z2 = [Z2;dx2'];
      end;
      Z1 = [Z1;x2(1:T-1,1:q1)'];
      Z1t = [Z1t;x2(2:T,1:q1)'];
   end;
   if q2>0;
      Z2 = [Z2;x3(2:T,:)'];
      if pe>0;
         Z2 = [Z2;x3(1:T-1,:)'];
      end;
   end;
else;
   Z0 = dx0(max(p,pe):T-1,:)';
   Z1 = x0(max(p,pe):T-1,:)';
   Z1t = x0(max(p,pe)+1:T,:)';
   for i=1:p-1;
      Z2 = [Z2;dx0(max(p,pe)-i:T-1-i,:)'];
   end;
   if q1>0;
      Z1 = [Z1;x2(max(p,pe):T-1,1:q1)'];
      Z1t = [Z1t;x2(max(p,pe)+1:T,1:q1)'];
      for i=0:pe-1;
         Z2 = [Z2;dx2(max(p,pe)-i:T-1-i,:)'];
      end;
   end;
   if q2>0;
      for i=0:pe;
         Z2 = [Z2;x3(max(p,pe)+1-i:T-i,:)'];
      end;
   end;
end;

%
% end of function vecmdata.m
%

