%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes Wald tests for Granger noncausality          %
%          conditional on beta - assumes chi^2 limiting          %
%          distribution, requires r>0 or p>1                     %
%                                                                %
%          Function: waldgctest                                  %
%                                                                %
%          Input:                                                %
%                       Psi: nxnq matrix                         %
%                       alpha: nxr matrix                        %
%                       beta: n+q1+dcointxr matrix with          %
%                           cointegration vectors                %
%                       p: number of lags for endogenous         %
%                           variables                            %
%                       pe: number of lags for exogenous         %
%                           variables                            %
%                       epsilon: nxT matrix with residuals       %
%                       Z1: nqxT matrix                          %
%                       Z2: nrxT matrix                          %
%                       q1: number of exogenous I(1) variables   %
%                       q0: number of exogenous I(0) variables   %
%                       d: number of unrestricted deterministic  %
%                           variables                            %
%                       t: degrees of freedom from original      %
%                           estimates                            %
%                       ind: 0-1 indicator to use restrictions   %
%                           on alpha under the null and the      %
%                           alternative (default=0)              %
%                       moddir: path to G-modelfile.rest         %
%                           files (default=empty string)         %
%                       modfile: name of model (default is       %
%                           empty string)                        %
%                       Sigma_theta: covariance matrix for Psi   %
%                           and alpha (optional)                 %
%                       c_ind: indicator if restrictions         %
%                           on C have been used                  %
%                                                                %
%          Output:                                               %
%          	            results: matrix with results             %
%                       df: matrix with degrees of freedom       %
%                                                                %
%          This version:        January 20, 2005                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2005          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [results,df] = waldgctest(Psi,alpha,beta,p,pe,epsilon,Z1,Z2,q1,q0,d,t,ind,moddir,modfile,Sigma_theta,c_ind)

if nargin<13;
   ind = 0;
   moddir = '';
   modfile = '';
   Sigma_theta = [];
end;

if nargin<17;
   c_ind = 0;
end;

results = [];
df = [];
numres = [];
actual_res = [];

curr_ini = find_inisettings;

%
% Preliminaries
%

[n T] = size(epsilon);

if isempty(alpha)==0;
   r = size(alpha,2);
   w = [Z2;beta'*Z1];
   beta_y = beta(1:n,:);
else;
   r = 0;
   w = Z2;
   beta_y = [];
end;
nreg = size(w,1);

Omega = (1/T)*epsilon*epsilon';
theta = vec([Psi alpha*beta_y']);

if ind==0;
   inv_ww = eye(nreg)/(w*w');
   if c_ind==0;
      Sigma_theta = kron(inv_ww,Omega);
   end;
   Rmat = [];
else;
   %
   % Get the restrictions file
   %
   G = [];
   file = strcat(moddir,'G-',int2str(n),'x',int2str(r),'-',modfile,'.rest');
   if fileexist(file)==1;
      G = loaddata(file);
   end;
   if isempty(G)==0;
      [Rmat,rvec] = rmatrix(G,n,r);
      dfa = size(G,1);
      if nreg>r;
         Rmat = [zeros(dfa,(nreg-r)*n) Rmat];
      end;
   else;
      Rmat = [];
   end;
end;

if c_ind==1;
   file = strcat(moddir,'C-',int2str(n),'x',int2str(r),'-',modfile,'.rest');
   F = loaddata(file);
   [Rmat_C,rvec_C] = rmatrix(F,n,n);
   if r<n;
      C = cmat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0);
   else;
      C = cmat_stat(Psi,alpha,beta,epsilon,Z1,Z2,p,d,0,[]);
   end;
   xi = xi_mat(C,Psi,alpha,p,pe,r,d,q1,q0);
   Rmat_C = Rmat_C*kron(xi',C);
   Rmat = [Rmat;Rmat_C];
   if rank(Rmat)<size(Rmat,1);
      [red_rest,use_rest] = getdelvars(Rmat',[]);
      Rmat = Rmat(use_rest,:);
   end;
   dfa = size(Rmat,1);
end;

if nreg>r;
   if r>0;
      fixpi = [[eye(n*(nreg-r)) zeros(n*(nreg-r),n*r)];[zeros(n*n,n*(nreg-r)) kron(beta_y,eye(n))]];
   else;
      fixpi = [eye(n*nreg);zeros(n*n,n*nreg)];
   end;
else;
   if r>0;
      fixpi = kron(beta_y,eye(n));
   end;
end;

Sigma_pi = fixpi*Sigma_theta*fixpi';

%
% Setting up the tests
%

In = eye(n);

for i=1:n;
   for k=1:n;
      if isequal(i,k)==0
         Nk = [];
         ei = In(:,i);
         ek = In(:,k);
         %
         % Setting up the matrix with restrictions
         %
         if p>1;
            if d>0;
               Nk = zeros(d,p-1);
            end;
            Nk = [Nk;kron(eye(p-1),ek)];
            if q1>0;
               Nk = [Nk;zeros(pe*q1,p-1)];
            end;
            if q0>0;
               Nk = [Nk;zeros((pe+1)*q0,p-1)];
            end;
         else;
            Nk = [];
         end;
         Nklev = Nk;
         if r>0;
            Np = [];
            Nplev = Np;
%            Nklev = Nk;
            if d>0;
               Np = zeros(d,1);
               Nplev = Np;
            end;
            if p>1;
               Nk = [Nk;zeros(r,p-1)];
               Np = [Np;zeros(n*(p-1),1)];
               Nklev = [Nklev;zeros(n,p-1)];
               Nplev = [Nplev;zeros(n*(p-1),1)];
            end;
            if q1>0;
               Np = [Np;zeros(pe*q1,1)];
               Nplev = [Nplev;zeros(pe*q1,1)];
            end;
            if q0>0;
               Np = [Np;zeros((pe+1)*q0,1)];
               Nplev = [Nplev;zeros((pe+1)*q0,1)];
            end;
            if max(abs(beta_y'*ek))>1.0e-005;
               Np = [Np;beta_y'*ek];
               Nplev = [Nplev;ek];
            else;
               Np = [];
               Nplev = [];
            end;
            Nk = [Nk Np];
            Nklev = [Nklev Nplev];
         end;
         R = kron(Nk',ei');
         Rlev = kron(Nklev',ei');
         if isempty(R)==1;
            W = 0;
         else;
            if r>0;
               if isempty(Rmat)==0;
                  [delrest,keeprest] = getdelvars([Rmat' R'],[]);
                  if size(keeprest,1)==dfa;
                     Rlev = [];
                     W = 0;
                  elseif size(keeprest,1)==dfa+size(R,1);
                     W = (Rlev*theta)'*inverse(Rlev*Sigma_pi*Rlev')*Rlev*theta;
                  else;
                     keeprest = keeprest(dfa+1:size(keeprest,1),1);
                     keeprest = keeprest-dfa;
                     Rlev = Rlev(keeprest,:);
                     W = (Rlev*theta)'*inverse(Rlev*Sigma_pi*Rlev')*Rlev*theta;
                  end;
               else;
                  W = (Rlev*theta)'*inverse(Rlev*Sigma_pi*Rlev')*Rlev*theta;
               end;
            else;
               Rlev = [Rlev zeros( size(Rlev,1) , size(Sigma_pi,1)-size(Rlev,2) )];
               theta = [theta;zeros(size(Sigma_pi,1)-size(theta,1),1)];
               W = (Rlev*theta)'*inverse(Rlev*Sigma_pi*Rlev')*Rlev*theta;
            end;
         end;
         results = [results;W];
         if size(Rlev,1)>0;
            numres = [numres;size(Rlev,1)];
         else;
            numres = [numres;1];
         end;
         actual_res = [actual_res;size(Rlev,1)];
      end;
   end;
end;

if curr_ini.sscorrect==0;
   df = actual_res;
   pval = chis_cdf(results,numres);
   results = [results 1-pval];
else;
   df = [actual_res t+actual_res];
   for i=1:size(numres,1);
      if numres(i,1)>0;
         results(i,1) = ((t+numres(i,1))/(T*numres(i,1)))*results(i,1);
      end;
   end;
   pval = fdis_cdf(results,numres,t+numres);
   results = [results 1-pval];
end;

%
% end of waldgctest.m
%
